//==================================================================================================
//
// This code can be used to compute an approximation for the spectral distorting caused by different
// energy releasee mechanisms. The computation is fast and based on the Greens function computed for
// a fixed background cosmology.
//
//==================================================================================================
//
// Author: Jens Chluba
// first implementation: April  2013
// last modification:    August 2013
//
//==================================================================================================

#ifndef GREENS_H
#define GREENS_H

//==================================================================================================
// Standards
//==================================================================================================
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <ctime>
#include <iomanip>
#include <cmath>
#include <limits.h>
#include <vector>

//==================================================================================================
// namespaces
//==================================================================================================
using namespace std;

const string Greens_version="Green's-function method v1.0.2";
const double T0_CMB=2.726;

//==================================================================================================
//
// set verbosity level
//
//==================================================================================================
void set_verbosity(int v);

//==================================================================================================
//
// some info from cosmology object
//
//==================================================================================================
double cosmos_Get_T0();
double cosmos_Get_time(double z);

//==================================================================================================
//
// to initialize the Greens functions
//
//==================================================================================================
void initialize_Greens_function();

//==================================================================================================
//
// to set special forms of the small-scale power spectrum
//
//==================================================================================================
void Set_P_step_params(double DA, double ks, double nSs);
void Set_P_bend_params(double kbend, double nSbend);
void Set_P_bend_params_B(double Abend, double nSbend);

//==================================================================================================
//
// main call of distortion functions
//
//==================================================================================================
void output_distortion(string fname,
                       double f_ann, int wave,
                       double A, double k0, double nS, double nrun,
                       double Gamma_X, double f_X,
                       double y, double Delta);

void output_Greens_function(string fname, double zs, double ze, int np);
void output_Greens_function_matrix(string fname, double zs, double ze, int np);

void output_y_visibility_function(string fname, double zs, double ze, int np);
void output_best_fit_mu_y_Decay(string fname, double fX_zX, double zs, double ze, int np);

//==================================================================================================
void compute_mock_data(double *nu_GHz, int np,
                       double f_ann, int wave,
                       double A, double k0, double nS, double nrun,
                       double Gamma_X, double f_X,
                       double y, double Delta, double Dnu_GHz, double DI_1sigma);

void compute_mock_data(string fname,
                       double f_ann, int wave,
                       double A, double k0, double nS, double nrun,
                       double Gamma_X, double f_X,
                       double y, double Delta,
                       double nuGHz_min, double nu_GHz_max, double Dnu_GHz, double DI_1sigma);

//==================================================================================================
//
// pre-binned version of the Green's function
//
//==================================================================================================
int initialize_Greens_function(double nu_GHz_min, double nu_GHz_max, double Dnu_GHz);

void compute_mock_data_binned(double *Sig, int np,
                              double f_ann, int wave,
                              double A, double k0, double nS, double nrun,
                              double Gamma_X, double f_X,
                              double y, double Delta, double DI_1sigma);

void compute_mock_data_binned(string fname,
                              double f_ann, int wave,
                              double A, double k0, double nS, double nrun,
                              double Gamma_X, double f_X,
                              double y, double Delta, double DI_1sigma);


//==================================================================================================
//
// save energy release history to file
//
//==================================================================================================
void output_energy_release_history(string fname,
                                   double f_ann, int wave,
                                   double A, double k0, double nS, double nrun,
                                   double Gamma_X, double f_X);

void evaluate_energy_release_history(double *zarr, int np,
                                     double f_ann, int wave,
                                     double A, double k0, double nS, double nrun,
                                     double Gamma_X, double f_X);

#endif

//==================================================================================================
//==================================================================================================
