r"""Compute spectral distortion for Decaying particle scenarios"""

import Greens as G
import numpy as np
import wrap_emcee
import h5py
import matplotlib.pyplot as mat

nthreadloc=4
usewalkers=500
nsamples  =5000000
addnoisetodata=0

#=======================================================================================
# many scenarios
#=======================================================================================
outnamemany="./Decay_scenarios/Decay.fac_1.fX_zX_5.0"

PIXIE_fac=1.0
f_G_X_model=5.0
rescalev3=1.0

#=======================================================================================
# scenario sequence
#=======================================================================================
outname="Decay.G_test.f_z_X_5.fac_1.hd5"
f_G_Xlabel="f_{\\rm X}/z_{\\rm X}\\,[ {\\rm eV } ]"
Gammalabel="z_{\\rm X}\\,[ 10^{4} ]"

PIXIE_fac=1.0

Gamma_model=23403.4731 #3.0e+5
rescalev4=1.0e+4

f_G_X_model=5.0
rescalev3=1.0

#=======================================================================================
# scenario large
#=======================================================================================
outname="Decay.G_1.0e+4.f_z_X_1.fac_10.nu_10.Dnu_5.hd5"
f_G_Xlabel="f_{\\rm X}/z_{\\rm X}\\,[ {\\rm eV } ]"
Gammalabel="z_{\\rm X}\\,[ 10^{4} ]"

PIXIE_fac=10.0

Gamma_model=5.0e+4
rescalev4=1.0e+4

f_G_X_model=5.0e+4/Gamma_model
rescalev3=1.0

#=======================================================================================
# test PCA paper
#=======================================================================================
outname="./Decay.MCMC/Decay.G_3.0e+5.narrow_prior.f_z_X_1.fac_5.hd5"
f_G_Xlabel="f_{\\rm X}/z_{\\rm X}\\,[ {\\rm eV } ]"
Gammalabel="z_{\\rm X}\\,[ 10^{5} ]"

PIXIE_fac=5.0

Gamma_model=3.0e+5
rescalev4=1.0e+5

f_G_X_model=1.0
rescalev3=1.0

#=======================================================================================
# scenario large
#=======================================================================================
outname="Decay.G_5.0e+4.f_z_X_10.fac_4.hd5"
f_G_Xlabel="f_{\\rm X}/z_{\\rm X}\\,[ {\\rm eV } ]"
Gammalabel="z_{\\rm X}\\,[ 10^{4} ]"

PIXIE_fac=4.0

Gamma_model=5.0e+4
rescalev4=1.0e+4

f_G_X_model=5.0e+5/Gamma_model
rescalev3=1.0

#=======================================================================================
# scenario small
#=======================================================================================
outname="Decay.G_5.0e+4.f_z_X_0.2.fac_4.B.hd5"
f_G_Xlabel="f_{\\rm X}/z_{\\rm X}\\,[ {\\rm eV } ]"
Gammalabel="z_{\\rm X}\\,[ 10^{4} ]"

PIXIE_fac=4.0

Gamma_model=5.0e+4
rescalev4=1.0e+4

f_G_X_model=1.0e+4/Gamma_model
rescalev3=1.0

#=======================================================================================
# common variables
#=======================================================================================
f_ann=0.0
wave=0

DeltaT=1.2e-4
rescalev1=1.0e-9

yre_model=4.0e-7
rescalev2=1.0e-7

priorv1 =np.array([-DeltaT*10.0,  DeltaT*10.0])/rescalev1
priorv2 =np.array([0.0,  yre_model*30.0])     /rescalev2
#priorv3 =np.array([0.0, f_G_X_model*30.0])    /rescalev3
#priorv3 =np.array([f_G_X_model/10.0, f_G_X_model*10.0])/rescalev3
priorv3 =np.array([f_G_X_model/3.0, f_G_X_model*3.0])/rescalev3
#priorv4 =np.array([1.1e+3, 4.9e+6])            /rescalev4
#priorv4 =np.array([5.0e+3, 3.0e+5])            /rescalev4
priorv4 =np.array([Gamma_model/3.0, Gamma_model*3.0])/rescalev4
#priorv4 =np.array([1.1e+3, Gamma_model*10.0])   /rescalev4


priorv3 =np.array([f_G_X_model/1.3, f_G_X_model*1.3])/rescalev3
priorv4 =np.array([Gamma_model/1.3, Gamma_model*1.3])/rescalev4

priorv3 =np.array([f_G_X_model/4.0, f_G_X_model*4.0])/rescalev3
priorv4 =np.array([Gamma_model/10.0, Gamma_model*4.0])/rescalev4


#=======================================================================================
def SD_model(bands=None, v1=None, v2=None, v3=None, v4=None):
    r"""Compute spectral distortion for Decaying particle + reionization scenario"""

    #===================================================================================
    # check prior
    #===================================================================================
    nf=len(bands)
    bands=np.ones(nf)*1.0e+10
 
    #print priorv3
    #print priorv4, "v4"

    if(v1 < priorv1[0]) or (v1 > priorv1[1]) or \
      (v2 < priorv2[0]) or (v2 > priorv2[1]) or \
      (v3 < priorv3[0]) or (v3 > priorv3[1])or \
      (v4 < priorv4[0]) or (v4 > priorv4[1]):
        return bands

    #===================================================================================
    Del  =v1*rescalev1
    y_re =v2*rescalev2
    f_G_X=v3*rescalev3
    z_X  =v4*rescalev4

    # decorrelate
    f_X=f_G_X*z_X

    #print "sample at: Delta = ", Del, " y_re = ", y_re, " f_X G_X = ", f_G_X, " z_X = ", z_X, " f_X = ", f_X

    G.compute_mock_data_binned(bands,
                               f_ann, wave,
                               0, 0, 0, 0,
                               z_X, f_X,
                               y_re, Del+DeltaT, 0)
    
    return bands


#=======================================================================================
def Decay_estimation(filename):
    r"""set up the parameters and run the chain"""

    #===================================================================================
    # frequencies
    #===================================================================================
    nu_GHz_min=30.0
    nu_GHz_max=1000.0
    Dnu_GHz=15.0
    
    #===================================================================================
    # initialize Greens function
    #===================================================================================
    nfreq=G.initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz)

    #===================================================================================
    # central frequencies
    #===================================================================================
    nu_GHzarr=np.array([nu_GHz_min+i*Dnu_GHz for i in range(nfreq)])
    
    #===================================================================================
    # mock data with noise
    #===================================================================================
    Sigarr=np.ones(nfreq)
    Garr=np.ones(nfreq)
    YSZarr=np.ones(nfreq)

    Del=DeltaT
    ypar=yre_model

    G.compute_mock_data_binned(Garr, 0, 0, 0, 0, 0, 0, 0, 0, 0, Del, 0)
    G.set_verbosity(-1)

    G.compute_mock_data_binned(YSZarr, 0, 0, 0, 0, 0, 0, 0, 0, ypar, 0, 0)

    f_X=f_G_X_model*Gamma_model
    Gamma=Gamma_model

    DI=5.0e-26 / PIXIE_fac
    
    G.compute_mock_data_binned(Sigarr, f_ann, wave, 0, 0, 0, 0, Gamma, f_X, ypar, Del, addnoisetodata*DI)
    
    #===================================================================================
    # energy history
    #===================================================================================
    G.output_energy_release_history(filename+".hist.dat",
                                    f_ann, wave,
                                    0, 0, 0, 0,
                                    Gamma, f_X);
    
    G.output_distortion(filename+".diss.dat",f_ann, wave,
                        0, 0, 0, 0, Gamma, f_X, 0, 0)
    
    #===================================================================================
    # errors and covariance
    #===================================================================================
    errarr=np.ones(nfreq)*DI*1.0e+26
    meas_cov = np.diag(errarr * errarr)

    #mat.plot(nu_GHzarr, Sigarr, 'r-')
    #mat.plot(nu_GHzarr, YSZarr,'b--')
    #mat.plot(nu_GHzarr, Garr,'r--')
    #mat.show()
    #print meas_cov
    
    #===================================================================================
    # MCMC parameters
    #===================================================================================
    default_params = {}
    default_params['bands'] = nu_GHzarr
    default_params['v1'] = 0.0         /rescalev1
    default_params['v2'] = yre_model   /rescalev2
    default_params['v3'] = f_G_X_model /rescalev3
    default_params['v4'] = Gamma_model /rescalev4

    Del_fit = {"kwarg_name": "v1",
               "kwarg_desc": "\\Delta^\\ast\\,[ 10^{-9} ]",
               "walker_range": priorv1,
               "prior_mean": 0.0,
               "prior_std": 0.0005/rescalev1
               }
    
    y_re_fit = {"kwarg_name": "v2",
                "kwarg_desc": "y_{\\rm re}\\,[ 10^{-7} ]",
                "walker_range": priorv2
               }

    f_G_X_fit = {"kwarg_name": "v3",
                 "kwarg_desc": f_G_Xlabel,
                 "walker_range": priorv3
                }

    G_X_fit = {"kwarg_name": "v4",
               "kwarg_desc": Gammalabel,
                "walker_range": priorv4
              }

    #===================================================================================
    # choose fit
    #===================================================================================
    fit_list = [Del_fit, y_re_fit, f_G_X_fit, G_X_fit]
    input_values = {}
    input_values['v1'] = 0
    input_values['v2'] = yre_model/rescalev2
    input_values['v3'] = f_G_X_model/rescalev3
    input_values['v4'] = Gamma_model/rescalev4

    wrap_emcee.call_mcmc(Sigarr, meas_cov, default_params, input_values,
                         fit_list, "Decay_fit_fX_zX.SD_model",
                         nwalkers=usewalkers, n_burn=500, n_run=nsamples/usewalkers,
                         threads=nthreadloc, verbose=False, outfile=filename)




#=======================================================================================
def Decay_estimation_many(filename):
    r"""set up the parameters and run the chain"""
    
    #===================================================================================
    # frequencies
    #===================================================================================
    nu_GHz_min=30.0
    nu_GHz_max=1000.0
    Dnu_GHz=15.0
    
    #===================================================================================
    # initialize Greens function
    #===================================================================================
    nfreq=G.initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz)
    G.set_verbosity(-1)
    
    #===================================================================================
    # central frequencies
    #===================================================================================
    nu_GHzarr=np.array([nu_GHz_min+i*Dnu_GHz for i in range(nfreq)])
    
    nmodels=5
    Df=(3.0e+5/1.0e+4)**(1.0/(nmodels-1))
    zXarr=np.array([1.0e+4*Df**i for i in range(nmodels)])
    variances3=np.ones(nmodels)
    best_fits3=np.ones(nmodels)
    variances4=np.ones(nmodels)
    best_fits4=np.ones(nmodels)
    
    #===================================================================================
    # mock data with noise
    #===================================================================================
    Sigarr=np.ones(nfreq)
    
    Del=DeltaT
    ypar=yre_model
    
    #===================================================================================
    # errors and covariance
    #===================================================================================
    DI=5.0e-26 / PIXIE_fac
    errarr=np.ones(nfreq)*DI*1.0e+26
    meas_cov = np.diag(errarr * errarr)
    
    #===================================================================================
    # MCMC parameters
    #===================================================================================
    default_params = {}
    default_params['bands'] = nu_GHzarr
    default_params['v1'] = 0.0            /rescalev1
    default_params['v2'] = yre_model      /rescalev2
    default_params['v3'] = f_G_X_model    /rescalev3
    
    Del_fit = {"kwarg_name": "v1",
        "kwarg_desc": "\\Delta^\\ast\\,[ 10^{-9} ]",
        "walker_range": priorv1,
        "prior_mean": 0.0,
        "prior_std": 0.0005/rescalev1
    }
    
    y_re_fit = {"kwarg_name": "v2",
        "kwarg_desc": "y_{\\rm re}\\,[ 10^{-7} ]",
        "walker_range": priorv2
        }

    f_G_X_fit = {"kwarg_name": "v3",
                 "kwarg_desc": f_G_Xlabel,
                 "walker_range": priorv3
                 }

    #===================================================================================
    # computation
    #===================================================================================
    for i in range(nmodels):
        
        zX=zXarr[i]
        v4start=np.array([zX/5.0, zX*5.0])/rescalev4

        print i, zX
        
        #global priorv4
        #priorv4[0]=zX/10.0/rescalev4
        #priorv4[1]=zX*10.0/rescalev4
        #print priorv4
        
        G_X_fit = {"kwarg_name": "v4",
                   "kwarg_desc": Gammalabel,
                   "walker_range": v4start,
                   "prior_mean": zX,
                   "prior_std": zX/rescalev1
                       }

        fname=filename+"."+str(i)+".hd5"
        
        G.compute_mock_data_binned(Sigarr, 0, 0, 0, 0, 0, 0, zX, f_G_X_model*zX, ypar, Del, addnoisetodata*DI)
         
        #===================================================================================
        # energy history
        #===================================================================================
        G.output_energy_release_history(fname+".hist.dat",
                                        0, 0,
                                        0, 0, 0, 0,
                                        zX, f_G_X_model*zX);

        G.output_distortion(fname+".diss.dat", 0, 0,
                            0, 0, 0, 0, zX, f_G_X_model*zX, 0, 0)

        #===================================================================================
        # MCMC parameters
        #===================================================================================
        default_params['v4'] = zX/rescalev4
        
        #===================================================================================
        # choose fit
        #===================================================================================
        fit_list = [Del_fit, y_re_fit, f_G_X_fit, G_X_fit]
        
        wrap_emcee.call_mcmc(Sigarr, meas_cov, default_params,
                             fit_list, "Decay_fit_fX_zX.SD_model",
                             nwalkers=usewalkers, n_burn=4000, n_run=nsamples/usewalkers,
                             threads=nthreadloc, verbose=False, outfile=fname)
        
        mcmc_data = h5py.File(fname, "r")
        chain_data = mcmc_data['chain']
        
        best_fits3[i]=np.mean(chain_data['v3'].value)
        variances3[i]=np.std(chain_data['v3'].value)
    
        best_fits4[i]=np.mean(chain_data['v4'].value)
        variances4[i]=np.std(chain_data['v4'].value)


    print "Results"
    
    for i in range(nmodels):
        print zXarr[i], best_fits3[i]*rescalev3, variances3[i], variances3[i]/best_fits3[i]

    print " zX:"

    for i in range(nmodels):
        print zXarr[i], best_fits4[i]*rescalev4, variances4[i], variances4[i]/best_fits4[i]


#=======================================================================================
#
# main call of code
#
#=======================================================================================
np.seterr(all='ignore')
if __name__ == '__main__':
    Decay_estimation(outname)
#Decay_estimation_many(outnamemany)


#=======================================================================================
#=======================================================================================
