r"""Compute spectral distortion for acoustic mode scenarios with small-scale step"""

import Greens as G
import numpy as np
import wrap_emcee
import h5py
import matplotlib.pyplot as mat

nthreadloc=4
usewalkers=200
nsamples  =200000
addnoisetodata=0

#=======================================================================================
# standard scenario
#=======================================================================================
outname="Dissipation.bend.nb_1.5.kb_3.B.fac_10.hd5"
#outname="Dissipation.bend.nb_2.0.kb_3.B.fac_100.long.hd5"
nblabel="n_{\\rm b}"
kblabel="k_{\\rm b}"
Ablabel="A_{\\rm b} [ 10^{-9} ]"

PIXIE_fac=10.0

nb_model=1.5
#nb_model=2.0
rescalev3=1.0

kb_model=3.0
rescalev4=1.0

#Ab_model=1.9515e-9*(45.0)**(nb_model-1.0)*(kb_model)**(0.96-nb_model)
Ab_model=7.2e-9
#Ab_model=2.8e-8
rescalev4A=1.0e-9

#=======================================================================================
# common variables
#=======================================================================================
Gamma=1.0
f_X=0.0
f_ann=0.0
wave=0
k0=0.05
A_back=2.2e-9
nS_back=0.96

DeltaT=1.2e-4
rescalev1=1.0e-9

yre_model=4.0e-7
rescalev2=1.0e-7

priorv1 =np.array([-1.0e-8, 1.0e-8])           /rescalev1
priorv2 =np.array([3.0e-7,  5.0e-7])           /rescalev2
priorv3 =np.array([nb_model/2.0, nb_model*2.0])/rescalev3
priorv4 =np.array([kb_model/10.0, kb_model*10.0])/rescalev4
priorv4A =np.array([Ab_model/100.0, Ab_model*10.0])/rescalev4A
#priorv4A =np.array([Ab_model/10.0, A_back*0.887072*(45.0)**(nb_model-1.0)])/rescalev4A

#=======================================================================================
def SD_model(bands=None, v1=None, v2=None, v3=None, v4=None):
    r"""Compute spectral distortion for acoustic modes scenario"""

    #===================================================================================
    # check prior
    #===================================================================================
    nf=len(bands)
    bands=np.ones(nf)*1.0e+10
    
    if(v1 < priorv1[0]) or (v1 > priorv1[1]) or \
      (v2 < priorv2[0]) or (v2 > priorv2[1]) or \
      (v3 < priorv3[0]) or (v3 > priorv3[1]) or \
      (v4 < priorv4[0]) or (v4 > priorv4[1]):
        return bands

    #===================================================================================
    Del  =v1*rescalev1
    y_re =v2*rescalev2
    nb   =v3*rescalev3
    kb   =v4*rescalev4

    G.Set_P_bend_params(kb, nb);

    print "sample at: Delta = ", Del, " y_re = ", y_re, " nb = ", nb, " kb = ", kb

    G.compute_mock_data_binned(bands,
                               f_ann, wave,
                               A_back, k0, nS_back, 0,
                               Gamma, f_X,
                               y_re, Del+DeltaT, 0)
    
    return bands


#=======================================================================================
def Dissipation_estimation(filename):
    r"""set up the parameters and run the chain"""

    #===================================================================================
    # frequencies
    #===================================================================================
    nu_GHz_min=30.0
    nu_GHz_max=1000.0
    Dnu_GHz=15.0
    
    #===================================================================================
    # initialize Greens function
    #===================================================================================
    nfreq=G.initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz)

    #===================================================================================
    # central frequencies
    #===================================================================================
    nu_GHzarr=np.array([nu_GHz_min+i*Dnu_GHz for i in range(nfreq)])
    
    #===================================================================================
    # mock data with noise
    #===================================================================================
    Sigarr=np.ones(nfreq)
    Garr=np.ones(nfreq)
    YSZarr=np.ones(nfreq)

    Del=DeltaT
    ypar=yre_model

    G.compute_mock_data_binned(Garr, 0, 0, 0, 0, 0, 0, 0, 0, 0, Del, 0)
    G.set_verbosity(-1)

    G.compute_mock_data_binned(YSZarr, 0, 0, 0, 0, 0, 0, 0, 0, ypar, 0, 0)

    DI=5.0e-26 / PIXIE_fac
    
    G.Set_P_bend_params(kb_model, nb_model);
    
    G.compute_mock_data_binned(Sigarr,
                               f_ann, wave,
                               A_back, k0, nS_back, 0,
                               Gamma, f_X,
                               ypar, Del, addnoisetodata*DI)
    
    #===================================================================================
    # energy history
    #===================================================================================
    G.output_energy_release_history(filename+".hist.dat",
                                    f_ann, wave,
                                    A_back, k0, nS_back, 0,
                                    Gamma, f_X);
    
    G.output_distortion(filename+".diss.dat",
                        f_ann, wave,
                        A_back, k0, nS_back, 0,
                        Gamma, f_X,
                        0, 0)
    
    #===================================================================================
    # errors and covariance
    #===================================================================================
    errarr=np.ones(nfreq)*DI*1.0e+26
    meas_cov = np.diag(errarr * errarr)

    #mat.plot(nu_GHzarr, Sigarr, 'r-')
    #mat.plot(nu_GHzarr, YSZarr,'b--')
    #mat.plot(nu_GHzarr, Garr,'r--')
    #mat.show()
    #print meas_cov
    
    #===================================================================================
    # MCMC parameters
    #===================================================================================
    default_params = {}
    default_params['bands'] = nu_GHzarr
    default_params['v1'] = 0.0              /rescalev1
    default_params['v2'] = yre_model        /rescalev2
    default_params['v3'] = nb_model         /rescalev3
    default_params['v4'] = kb_model         /rescalev4

    Del_fit = {"kwarg_name": "v1",
               "kwarg_desc": "\\Delta^\\ast\\,[ 10^{-9} ]",
               "walker_range": priorv1,
               "prior_mean": 0.0,
               "prior_std": 0.0005/rescalev1
               }
    
    y_re_fit = {"kwarg_name": "v2",
                "kwarg_desc": "y_{\\rm re}\\,[ 10^{-7} ]",
                "walker_range": priorv2
               }

    nb_fit = {"kwarg_name": "v3",
                 "kwarg_desc": nblabel,
                 "walker_range": priorv3
                }

    kb_fit = {"kwarg_name": "v4",
               "kwarg_desc": kblabel,
               "walker_range": priorv4
              }

    #===================================================================================
    # choose fit
    #===================================================================================
    fit_list = [Del_fit, y_re_fit, nb_fit, kb_fit]

    wrap_emcee.call_mcmc(Sigarr, meas_cov, default_params,
                         fit_list, "Dissipation_bend.SD_model",
                         nwalkers=usewalkers, n_burn=500, n_run=nsamples/usewalkers,
                         threads=nthreadloc, verbose=False, outfile=filename)


#=======================================================================================
def SD_model_B(bands=None, v1=None, v2=None, v3=None, v4=None):
    r"""Compute spectral distortion for acoustic modes scenario"""
    
    #===================================================================================
    # check prior
    #===================================================================================
    nf=len(bands)
    bands=np.ones(nf)*1.0e+10
    
    p4=priorv4A[1]
    #nb=v3*rescalev3
    #p4=A_back*0.887072*(45.0)**(nb_model-1.0)
    
    if(v1 < priorv1[0]) or (v1 > priorv1[1]) or \
        (v2 < priorv2[0]) or (v2 > priorv2[1]) or \
        (v3 < priorv3[0]) or (v3 > priorv3[1]) or \
        (v4 < priorv4A[0]) or (v4 > p4):
            return bands
    
    #===================================================================================
    Del  =v1*rescalev1
    y_re =v2*rescalev2
    Ab   =v4*rescalev4A
    nb=v3*rescalev3

    G.Set_P_bend_params_B(Ab, nb);
    
    #print "sample at: Delta = ", Del, " y_re = ", y_re, " nb = ", nb, " Ab = ", Ab
    
    G.compute_mock_data_binned(bands,
                               f_ann, wave,
                               A_back, k0, nS_back, 0,
                               Gamma, f_X,
                               y_re, Del+DeltaT, 0)
    
    return bands


#=======================================================================================
def Dissipation_estimation_B(filename):
    r"""set up the parameters and run the chain"""
    
    #===================================================================================
    # frequencies
    #===================================================================================
    nu_GHz_min=30.0
    nu_GHz_max=1000.0
    Dnu_GHz=15.0
    
    #===================================================================================
    # initialize Greens function
    #===================================================================================
    nfreq=G.initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz)
    
    #===================================================================================
    # central frequencies
    #===================================================================================
    nu_GHzarr=np.array([nu_GHz_min+i*Dnu_GHz for i in range(nfreq)])
    
    #===================================================================================
    # mock data with noise
    #===================================================================================
    Sigarr=np.ones(nfreq)
    Garr=np.ones(nfreq)
    YSZarr=np.ones(nfreq)
    
    Del=DeltaT
    ypar=yre_model
    
    G.compute_mock_data_binned(Garr, 0, 0, 0, 0, 0, 0, 0, 0, 0, Del, 0)
    G.set_verbosity(-1)
    
    G.compute_mock_data_binned(YSZarr, 0, 0, 0, 0, 0, 0, 0, 0, ypar, 0, 0)
    
    DI=5.0e-26 / PIXIE_fac
    
    G.Set_P_bend_params_B(Ab_model, nb_model);
    
    G.compute_mock_data_binned(Sigarr,
                               f_ann, wave,
                               A_back, k0, nS_back, 0,
                               Gamma, f_X,
                               ypar, Del, addnoisetodata*DI)
    
    #===================================================================================
    # energy history
    #===================================================================================
    G.output_energy_release_history(filename+".hist.dat",
                                    f_ann, wave,
                                    A_back, k0, nS_back, 0,
                                    Gamma, f_X);
    
    G.output_distortion(filename+".diss.dat",
                        f_ann, wave,
                        A_back, k0, nS_back, 0,
                        Gamma, f_X,
                        0, 0)
    
    #===================================================================================
    # errors and covariance
    #===================================================================================
    errarr=np.ones(nfreq)*DI*1.0e+26
    meas_cov = np.diag(errarr * errarr)
    
    #===================================================================================
    # MCMC parameters
    #===================================================================================
    default_params = {}
    default_params['bands'] = nu_GHzarr
    default_params['v1'] = 0.0              /rescalev1
    default_params['v2'] = yre_model        /rescalev2
    default_params['v3'] = nb_model         /rescalev3
    default_params['v4'] = Ab_model         /rescalev4A
    
    Del_fit = {"kwarg_name": "v1",
        "kwarg_desc": "\\Delta^\\ast\\,[ 10^{-9} ]",
        "walker_range": priorv1,
        "prior_mean": 0.0,
        "prior_std": 0.0005/rescalev1
    }
    
    y_re_fit = {"kwarg_name": "v2",
        "kwarg_desc": "y_{\\rm re}\\,[ 10^{-7} ]",
        "walker_range": priorv2
        }
    
    nb_fit = {"kwarg_name": "v3",
        "kwarg_desc": nblabel,
        "walker_range": priorv3
        }
    
    Ab_fit = {"kwarg_name": "v4",
        "kwarg_desc": Ablabel,
        "walker_range": priorv4A
        }
    
    #===================================================================================
    # choose fit
    #===================================================================================
    fit_list = [Del_fit, y_re_fit, nb_fit, Ab_fit]
    
    input_values = {}
    input_values['v1'] = 0
    input_values['v2'] = yre_model /rescalev2
    input_values['v3'] = nb_model/rescalev3
    input_values['v4'] = Ab_model/rescalev4

    wrap_emcee.call_mcmc(Sigarr, meas_cov, default_params, input_values,
                         fit_list, "Dissipation_bend.SD_model_B",
                         nwalkers=usewalkers, n_burn=500, n_run=nsamples/usewalkers,
                         threads=nthreadloc, verbose=False, outfile=filename)


#=======================================================================================
#
# main call of code
#
#=======================================================================================
np.seterr(all='ignore')
if __name__ == '__main__':
    #Dissipation_estimation(outname)
    Dissipation_estimation_B(outname)


#=======================================================================================
#=======================================================================================

