//==================================================================================================
//
// basic Greens python functions
// 
//==================================================================================================
//
// Author: Jens Chluba
//
// first implementation:  April 2013
// last modification   :  April 2013
//
//==================================================================================================

#ifndef GREENS_PY_H
#define GREENS_PY_H

#include <iostream>
#include <string>

using namespace std;

//==================================================================================================
//
// set verbosity level
//
//==================================================================================================
void set_verbosity(int v);

//==================================================================================================
//
// some info from cosmology object
//
//==================================================================================================
double cosmos_Get_T0();
double cosmos_Get_time(double z);

//==================================================================================================
//
// to initialize the Greens functions
//
//==================================================================================================
void initialize_Greens_function();

//==================================================================================================
//
// to set special forms of the small-scale power spectrum
//
//==================================================================================================
void Set_P_step_params(double DA, double ks, double nSs);
void Set_P_bend_params(double kbend, double nSbend);
void Set_P_bend_params_B(double Abend, double nSbend);

//==================================================================================================
//
// main call of distortion functions
//
//==================================================================================================
void output_distortion(string fname,
                       double f_ann, int wave,
                       double A, double k0, double nS, double nrun,
                       double Gamma_X, double f_X,
                       double y, double Delta);

void compute_mock_data(double *nu_GHz, int np,
                       double f_ann, int wave,
                       double A, double k0, double nS, double nrun,
                       double Gamma_X, double f_X,
                       double y, double Delta, double Dnu_GHz, double DI_1sigma);

//==================================================================================================
//
// pre-binned version of the Green's function
//
//==================================================================================================
int initialize_Greens_function(double nu_GHz_min, double nu_GHz_max, double Dnu_GHz);

void compute_mock_data_binned(double *Sig, int np,
                              double f_ann, int wave,
                              double A, double k0, double nS, double nrun,
                              double Gamma_X, double f_X,
                              double y, double Delta, double DI_1sigma);

void compute_mock_data_binned(string fname,
                              double f_ann, int wave,
                              double A, double k0, double nS, double nrun,
                              double Gamma_X, double f_X,
                              double y, double Delta, double DI_1sigma);

//==================================================================================================
//
// save energy release history to file
//
//==================================================================================================
void output_energy_release_history(string fname,
                                   double f_ann, int wave,
                                   double A, double k0, double nS, double nrun,
                                   double Gamma_X, double f_X);

void evaluate_energy_release_history(double *zarr, int np,
                                     double f_ann, int wave,
                                     double A, double k0, double nS, double nrun,
                                     double Gamma_X, double f_X);

#endif

//==================================================================================================
//==================================================================================================
